function elapsed = newton3d()
% Celestial mechanics with MATLAB
% The program computes the dynamics of a satellite
% around the Lagrange point L_4 in the Earth-Moon system
%
% Units: [length] = 10^9 m, [time]  =  day (24*3600 s)
%
% The satellite is located at L_4, the Lagrange point with a 
% random initial velocity w.r.t. the drag velocity of L_4.
%
% Configuration can be saved and reloaded
%
% Variables: positions with respect to the Earth-Moon center of
% mass (xCM) are denoted by xE (Earth), xM (Moon), xS (satellite);
% velocities are denoted by the same symbol with a "d" added like xEd. 
% On startup the program lets you choose whether 
% 1) plotting data from a previous run, 
% 2) resume from a previous run, 
% 3) start a new run.
% Parameters can be changed via the initial dialog box. 
% The dynamics of Sun-Earth-Moon alone can be studied by 
% selecting 'n' in the 'launch satellite' field.
% See "A tutorial on celestial mechanics using Matlab", 
% Computingn in Science and Engineering, November 2001
% (C) Enrico Onofri, 2000-2001
% This program can be freely copied and used for non-profit
% (educational) purposes. Any change to the original source code
% should be clearly identified prior to distribution. 
% files: 
% three_body3d.m, periods.m, earth_moon3d.m, collision.m,
% frequencies.m

global GMM	% Newton's constant x Moon's mass
global GME	% Newton's constant x Earth's mass
global GM       % Newton's constant x Sun's mass

AU     =  149.6;        % Astronomical Unit: earth-sun mean distance
DEM    =  0.3844;	% moon-earth                     "     "
Year   =  365.2564;	
Month  =  27.32166;	% sidereal month
%=  27.55455            % perigee to perigee
%=  29.5306             % synodic (full moon to full moon)
%=  27.212221           % draconitic

selection  =  {'Plotting','Resume','New run'};
choice     =  menu('Start',selection);

entries = {
    'Earth/Moon mass ratio (\mu)',... 
    'earth''s orbit eccentricity',...
    'moon''s orbit mean eccentricity', ...
    'G M_{\o}',...
    'perturb(%)',...
    'launch satellite? (y/n)',...
    'tolerance',...
    'time unit',...
    'T_{max}',...
    'number of snapshots',...
    'moon''s orbit inclination',...
    'save results? (y/n)',...
    'save file name',...
    'plot in real time? (y/n)'
};

switch choice
 case 1				% load and plot
  d = dir('*.mat');
  files = {d.name}';
  fprompt = {num2cell((1:max(size(files)))'), files};
  [fprompt{1,:}]
  fn = input('choose number > ');
  eval(['load ' files{fn}]);
  
  if(exist('xS') == 1),
    figure
    plot(T,sqrt(sum((xS-xE+xCM)'.^2)),T,sqrt(sum(xR'.^2)'),...
	 'LineWidth',2)
    grid on
    set(gca,'GridLineStyle',':','FontName','Courier','FontSize',14)
    xlabel('Time (days)')
    ylabel('Distance from Earth (MKm)')';
  end
  if(exist('NL') == 1 & exist('T') == 1),
    figure
    plot(T,sqrt(sum(NL'.^2)));
    set(gca,'GridLineStyle',':','FontName','Courier','FontSize',14)
    xlabel('Time (days)')
    ylabel('Moon''s mean eccentricity')
    figure
    plot(T,sqrt(sum(NCM'.^2)));
    set(gca,'GridLineStyle',':','FontName','Courier','FontSize',14)
    xlabel('Time (days)')
    ylabel('X_{CM}''s mean eccentricity')      
    periods(T,NL);
  end
  
 case 2				% resume a previous run
  d = dir('*.mat');		% are there any?
  files = {d.name};
  fprompt = {num2cell((1:max(size(files)))'), files'};
  [fprompt{1,:}]		% prompt the user
  fn = input('choose number > ');
  eval(['load ' files{fn}]);	% load previous run
  default = setup;
  
 case 3		     		% new run
  default = {
      '81.3016',...  % initial data - change only via dialog menu
      '0.0167',...    % DO NOT CHANGE SOURCE FILE
      '0.0549',...	 
      num2str(AU^3/(Year/(2*pi))^2),... % Kepler III
      '0.0',...
      'y',...
      '1e-12',...
      'Years',...
      '1',...
      '256',...
      '.0898',...			% 5deg 8'43"
      'y',...
      'sat3d',...
      'y'
 };
 otherwise
  disp('Wrong choice -start again');
  exit
end

if(choice == 1),
  break;
else,
  AddOpts.Interpreter='tex';
  AddOpts.Resize='on';
  AddOpts.WindowStyle='normal';
  
  title = 'Initial data';
  lines = 1;
  setup = inputdlg(entries,title,lines,default,AddOpts);
  
  QEM   =  str2num(setup{1});
  eT    =  str2num(setup{2});
  eL    =  str2num(setup{3});
  GM    =  str2num(setup{4});
  pert  =  str2num(setup{5})/100;
  
  if choice == 3,
    launch  =  setup{6};
  end
  
  tol   =  str2num(setup{7});
  unit  =  setup{8};
  tm    =  str2num(setup{9});
  
  if unit(1) == 'Y' | unit(1) == 'y',
    Tmax  =  tm*Year;
  elseif unit(1) == 'm' | unit(1) == 'M'
    Tmax  =  tm*Month;
  else 
    Tmax  =  str2num(unit)*tm;
  end
  
  dtm    =  str2num(setup{10});
  alpha  =  str2num(setup{11});
  rtplot =  setup{14};
  
  switch choice
   case 1
    break
   case 2		% resume from an old run
    n1=length(T); 
    T1=T(end);
    dT=dtm/(n1-1)*T1-Tmax; % preserve time step by adjusting Tmax
    Tprev = T(1:end-1);	   % save previous data
    Yprev = Y(1:end-1,:);			

    prev_step = Tprev(2)-Tprev(1);
    
    time = linspace(T1,T1+Tmax+dT,dtm+1);

    if(abs(time(2)-time(1)-prev_step)>tol), % not the same step
					    % no spectral analysis
      disp('Warning: time step size will not be uniform');
      disp('No spectral analysis can be performed');
    end
    
    if launch == 'y',	% launch satellite
      y = Y(end,:);			
      tic;
      [T,Y,TE,YE,IE] = ode113(@three_body3d,time,y,options);
      elapsed = toc;
      
      Y = [Yprev; Y];	% join old and new orbit data
      T = [Tprev; T];

      xCM = Y(:,1:3);
      xR = Y(:,4:6);
      xS = Y(:,7:9);
      
      [xM,xE,xMd,xEd] = check13d(T,Y);
      figure
      plot(T,sqrt(sum((xS-xE+xCM)'.^2)),T,sqrt(sum(xR'.^2)'),...
	  'LineWidth',2)
      grid on

      set(gca,'GridLineStyle',':','FontName','Courier','FontSize',14)
      ylabel('Distance from Earth (MKm)')
      xlabel('time (days)')

    else		% only earth and moon
      y = Y(end,:);
      tic;
      [T,Y] = ode113(@earth_moon3d,time,y,options);
      elapsed = toc;
      
      Y = [Yprev; Y];	% join old and new orbit data
      T = [Tprev; T];
      
      [xM,xE,xMd,xEd,xCM,xCMd] = check23d(T,Y);
      
      xR = xM-xE;
      xRd = xMd-xEd;
      % compute Laplace-Runge-Lenz vector       
      NL = xR./sqrt(inner3d(xR,xR)) - (inner3d(xRd,xRd).*xR - ...
				       inner3d(xRd,xR).*xRd)/GMEM;
      NCM = xCM./sqrt(inner3d(xCM,xCM)) - (inner3d(xCMd,xCMd).*xE - ...
					   inner3d(xCMd,xCM).*xCMd)/GM;
      figure
      plot(T,sqrt(sum(NL'.^2)));
      set(gca,'GridLineStyle',':','FontName','Courier','FontSize',14)
      xlabel('Time (days)')
      ylabel('Moon''s mean eccentricity')
      figure
      plot(T,sqrt(sum(NCM'.^2)));
      set(gca,'GridLineStyle',':','FontName','Courier','FontSize',14)
      xlabel('Time (days)')
      ylabel('X_{CM}''s mean eccentricity')
    end
    
   case 3				% new run
    Rincl = [1, 0, 0; 0, cos(alpha), -sin(alpha);... % inclination matrix
	     0, sin(alpha), cos(alpha)];
    GMEM = (DEM)^3/(Month/2/pi)^2;	% GME = .002975557 
    GMM = GMEM/(1+QEM);			% GMM = 3.659901e-5
    GME = GMEM-GMM;			% GMEM = 0.003012156
    
    xCM   =  AU*(1-eT)*[1;0;0];		% earth-moon center of mass
    xCMd  =  sqrt(GM*(1+eT)/(AU*(1-eT)))*[0;1;0]; % CM velocity at perielium
    
    xR  =  DEM*(1-eL)*[1;0;0];		% Moon-Earth vector
    xE  =  - xR/(QEM+1.);			% position of Earth
    xM  =  xR+xE;				% position of Moon 
					% relative to Center of Mass
    
					% "d" denotes "dot" i.e. d/dt
    xRd   =  sqrt(GMEM*(1+eL)/(DEM*(1-eL)))*[0;1;0]; % relative velocity
    
    xEd   =  -xRd/(QEM+1);		% vel w.r.t. CM (earth)
    xMd   =  xRd+xEd;			% vel w.r.t. CM (moon)
    
    time = linspace(0,Tmax,dtm);		% need regular snapshots
    
    if (launch == 'y'),			% launch satellite in L4
      tmp = DEM*(1-eL)*[0;1;0];
      xS  =  0.5*(xE+xM);	        % satellite position
      xS  =  xS + sqrt(3)/2*tmp;      
      
      xSd  =  zeros(3,1);		% satellite velocity
      xSd(1)  =  0.5*sqrt(3)*(xEd(2)-xMd(2));	
      xSd(2)  =  0.5*(xEd(2)+xMd(2)); 
      
      xRd = Rincl*xRd;			% rotate to the right inclination
      xS = Rincl*xS;
      xSd = Rincl*xSd;
      
      options = odeset('RelTol',tol,...
		       'AbsTol', tol*[
			   400*ones(3,1);...
			       ones(3,1); ...
			       ones(3,1); ...
			   10.0*ones(3,1); ...
			        ones(3,1); ...
			        ones(3,1)], ...
		       'OutputSel',[7 8 9],...	
		       'OutputFcn',@odephas3,... 
		       'refine',1,...
		       'Events',@collision);
      
      if(rtplot(1) ~= 'y' & rtplot(1) ~= 'Y'),
	options.OutputFcn = 'odeprint';
      end
      
      rand('state',sum(100*clock)); % give L4 a random velocity and momentum
      
      xSd =  xSd.*(ones(3,1)+pert*randn(3,1));
      xS  =   xS.*(ones(3,1)+pert*randn(3,1));
      figure;
      
      y  =  [
	  xCM;xR;xS;		% initial positions...
	  xCMd;xRd;xSd		% and velocities
	    ];
      
      tic;
      [T,Y,TE,YE,IE] = ode113(@three_body3d,time,y,options);
      elapsed = toc;		% elapsed time with this run

      xCM = Y(:,1:3);
      xR = Y(:,4:6);
      xS = Y(:,7:9);
      
      [xM,xE,xMd,xEd] = check13d(T,Y);
      figure
      plot(T,sqrt(sum((xS-xE+xCM)'.^2)),T,sqrt(sum(xR'.^2)'),'LineWidth',2)
      set(gca,'GridLineStyle',':','FontName','Courier','FontSize',14)
      ylabel('Distance from Earth (MKm)')
      xlabel('time (days)')

    else		% only earth and moon
      
      xRd = Rincl*xRd;
      options = odeset('RelTol',tol,...
		     'AbsTol', tol*[
			 400.0*ones(3,1);... % CM is >> other components
			       ones(3,1); ...
			  10.0*ones(3,1); ...
			   0.2*ones(3,1)], ...
		     'OutputSel',[4 5 6 ],...	% plot H1-H2 during evolution
		     'OutputFcn','odephas3',... % try also odephas2  
		     'refine',1);
         
      if(rtplot(1) ~= 'y' & rtplot(1) ~= 'Y'),
	options.OutputFcn = 'odeprint';
      end
      
      
      y  =  [
	  xCM;		% initial positions...
	  xR;			
	  xCMd;         % and velocities
	  xRd
	    ];			
      
      tic;
      [T,Y] = ode113(@earth_moon3d,time,y,options);
      elapsed = toc;
      
      [xM,xE,xMd,xEd,xCM,xCMd] = check23d(T,Y);
      
      xR = xM-xE;
      xRd = xMd-xEd;
      
      NL = xR./sqrt(inner3d(xR,xR)) - (inner3d(xRd,xRd).*xR - ...
	  inner3d(xRd,xR).*xRd)/GMEM;
      NCM = xCM./sqrt(inner3d(xCM,xCM)) - (inner3d(xCMd,xCMd).*xE - ...
	  inner3d(xCMd,xCM).*xCMd)/GM;
      figure
      subplot(211)
      plot(T,sqrt(sum(NL'.^2)));
      set(gca,'GridLineStyle',':','FontName','Courier','FontSize',14)
      xlabel('Time (days)')
      ylabel('Moon''s mean eccentricity')
      subplot(212)
      plot(T,sqrt(sum(NCM'.^2)));
      set(gca,'GridLineStyle',':','FontName','Courier','FontSize',14)
      xlabel('Time (days)')
      ylabel('X_{CM}''s mean eccentricity')
    end
   otherwise
    disp('This is an internal bug - please report to the author');
    disp('Mailto: onofri@pr.infn.it');
  end
  
  clear choice				% choice must be refreshed!
  final = setup{12};			% save history 
  if final(1) == 'y' | final(1) == 'Y'
    eval(['save ' setup{13}]);
    disp(['results saved to file ' setup{13} '.mat']);
  end
end

%---------------------   private subroutines 

function y = inner3d(a,b)

y  =  sum(a'.*b')';
y  =  y*[1 1 1];

%----------

function [xM,xE,xMd,xEd] = check13d(T,Y);

global GMM
global GME
global GM


cT = GME/(GME+GMM);
cL = 1-cT;

xCM    =  Y(:,1:3);
xR     =  Y(:,4:6);
xCMd   =  Y(:,10:12);
xRd    =  Y(:,13:15);

xEd  =  xCMd - cL*xRd;
xMd  =  xCMd + cT*xRd;
xE  =  xCM - cL*xR;
xM  =  xCM + cT*xR;

E_earth = GME/2*sum(xEd'.^2) - GME*GM./sqrt(sum(xE'.^2));
E_moon  = GMM/2*sum(xMd'.^2) - GMM*GM./sqrt(sum(xM'.^2));
E_tot   = E_earth + E_moon - GME*GMM./sqrt(sum(xR'.^2));

amT =  GME*(xE(:,1).*xEd(:,2)-xE(:,2).*xEd(:,1));
amL =  GMM*(xM(:,1).*xMd(:,2)-xM(:,2).*xMd(:,1));
figure
subplot(211)
plot(diff(E_tot')./diff(T))
subplot(212)
plot(diff(amT+amL)./diff(T))


%----------
function [xM,xE,xMd,xEd,xCM,xCMd] = check23d(T,Y);

global GMM
global GME
global GM

cT = GME/(GME+GMM);
cL = 1-cT;

xCM    =  Y(:,1:3);
xR     =  Y(:,4:6);
xCMd   =  Y(:,7:9);
xRd    =  Y(:,10:12);

xEd  =  xCMd - cL*xRd;
xMd  =  xCMd + cT*xRd;
xE  =  xCM - cL*xR;
xM  =  xCM + cT*xR;

E_earth = GME/2*sum(xEd'.^2) - GME*GM./sqrt(sum(xE'.^2));
E_moon = GMM/2*sum(xMd'.^2) - GMM*GM./sqrt(sum(xM'.^2));
E_tot = E_earth + E_moon - GME*GMM./sqrt(sum(xR'.^2));

amT  =  GME*(xE(:,1).*xEd(:,2)-xE(:,2).*xEd(:,1));
amL  =  GMM*(xM(:,1).*xMd(:,2)-xM(:,2).*xMd(:,1));
figure
subplot(211)
plot(diff(E_tot')./diff(T))
subplot(212)
plot(diff(amT+amL)./diff(T))



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Astronomical data for our planetary system (MKS)
% G_N  =  6.67423(9)e-11 m^3/kg s^2 (Newton's constant)
% AU  =  149.5978 (in Mkm)
% year = 31556925.2 s;
% M_Sun  =  1.98892e30 kg
% 
% Legend: semimajor axis (AU), orbital period, eccentricity, inclination in
% degrees, spin period, obliquity to the ecliptic in degrees
% mercury = [.387    87.97  0.206  7.00  58.65     0]
% venus   = [.723   224.7   0.007  3.39 243.0  178.0]
% earth   = [1.     365.2564  0.0167  0.0    1.00  23.4]
% mars    = [1.524  686.98  0.093  1.85   1.026 25.0]
% Jupiter = [5.203   11.86y 0.048  1.30   0.410 3.08]
% Saturn  = [9.539   29.46y 0.056  2.49   0.426 26.7]
% Uranus  = [19.182  84.01y 0.047  0.77   0.720 97.9]
% Neptune = [30.058 164.79y 0.009  1.77   0.670 28.8]
% Pluto   = [39.44  247.7y  0.250 17.2    6.387 94.0]

% Equat.radius (km), mass (g) mean density g/cm^3
% Sun:     695700 1.99e33 1.41
% mercury:   2439 3.30e26  5.42 
% venus:     6502 4.87e27 5.25
% earth:     6378 5.98e27 5.52
% mars:      3398 6.42e26 3.94
% jupiter:  71900 1.90e30  1.31
% saturn:   60330 5.69e29  .69
% uranus:   25700 8,68e28 1.22
% neptune:  24750 1.03e29 1.66
% pluto:     1100 1.22e28 2.1

% Moon: 384.4 (Mm) 27.322 (period) .0549 (eccentr.) 5.1 (inclination
% degrees) 1738 (radius) 3.34 (density)
%----------------------------------------------------------------------
% Taken from B. Bertotti and P. Farinella, "The Physics of the
% solar system", Kluwer Acad. Publ. 1990.
%----------------------------------------------------------------------
